@@ -12,6 +12,7 @@ use Cwd                         qw[cwd];
 use Carp                        qw[carp];
 use IPC::Cmd                    qw[can_run run QUOTE];
 use File::Path                  qw[mkpath];
+use File::Temp                  qw[tempdir];
 use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
 use Locale::Maketext::Simple    Style => 'gettext';
@@ -21,7 +22,7 @@ use vars    qw[ $VERBOSE $PREFER_BIN $FROM_EMAIL $USER_AGENT
                 $FTP_PASSIVE $TIMEOUT $DEBUG $WARN
             ];
 
-$VERSION        = '0.18';
+$VERSION        = '0.26';
 $VERSION        = eval $VERSION;    # avoid warnings with development releases
 $PREFER_BIN     = 0;                # XXX TODO implement
 $FROM_EMAIL     = 'File-Fetch@example.com';
@@ -35,7 +36,7 @@ $WARN           = 1;
 
 ### methods available to fetch the file depending on the scheme
 $METHODS = {
-    http    => [ qw|lwp wget curl lftp lynx| ],
+    http    => [ qw|lwp httplite wget curl lftp lynx iosock| ],
     ftp     => [ qw|lwp netftp wget curl lftp ncftp ftp| ],
     file    => [ qw|lwp lftp file| ],
     rsync   => [ qw|rsync| ]
@@ -177,13 +178,13 @@ result of $ff->output_file will be used.
         bless $args, $class;
     
         if( lc($args->scheme) ne 'file' and not $args->host ) {
-            return File::Fetch->_error(loc(
+            return $class->_error(loc(
                 "Hostname required when fetching from '%1'",$args->scheme));
         }
         
         for (qw[path file]) {
             unless( $args->$_() ) { # 5.5.x needs the ()
-                return File::Fetch->_error(loc("No '%1' specified",$_));
+                return $class->_error(loc("No '%1' specified",$_));
             }
         }
         
@@ -274,10 +275,10 @@ sub new {
     check( $tmpl, \%hash ) or return;
 
     ### parse the uri to usable parts ###
-    my $href    = __PACKAGE__->_parse_uri( $uri ) or return;
+    my $href    = $class->_parse_uri( $uri ) or return;
 
     ### make it into a FFI object ###
-    my $ff      = File::Fetch->_create( %$href ) or return;
+    my $ff      = $class->_create( %$href ) or return;
 
 
     ### return the object ###
@@ -397,10 +398,19 @@ sub _parse_uri {
     return $href;
 }
 
-=head2 $ff->fetch( [to => /my/output/dir/] )
+=head2 $where = $ff->fetch( [to => /my/output/dir/ | \$scalar] )
 
-Fetches the file you requested. By default it writes to C<cwd()>,
-but you can override that by specifying the C<to> argument.
+Fetches the file you requested and returns the full path to the file.
+
+By default it writes to C<cwd()>, but you can override that by specifying 
+the C<to> argument:
+
+    ### file fetch to /tmp, full path to the file in $where
+    $where = $ff->fetch( to => '/tmp' );
+
+    ### file slurped into $scalar, full path to the file in $where
+    ### file is downloaded to a temp directory and cleaned up at exit time
+    $where = $ff->fetch( to => \$scalar );
 
 Returns the full path to the downloaded file on success, and false
 on failure.
@@ -411,21 +421,31 @@ sub fetch {
     my $self = shift or return;
     my %hash = @_;
 
-    my $to;
+    my $target;
     my $tmpl = {
-        to  => { default => cwd(), store => \$to },
+        to  => { default => cwd(), store => \$target },
     };
 
     check( $tmpl, \%hash ) or return;
 
-    ### On VMS force to VMS format so File::Spec will work.
-    $to = VMS::Filespec::vmspath($to) if ON_VMS;
+    my ($to, $fh);
+    ### you want us to slurp the contents
+    if( ref $target and UNIVERSAL::isa( $target, 'SCALAR' ) ) {
+        $to = tempdir( 'FileFetch.XXXXXX', CLEANUP => 1 );
 
-    ### create the path if it doesn't exist yet ###
-    unless( -d $to ) {
-        eval { mkpath( $to ) };
+    ### plain old fetch
+    } else {
+        $to = $target;
 
-        return $self->_error(loc("Could not create path '%1'",$to)) if $@;
+        ### On VMS force to VMS format so File::Spec will work.
+        $to = VMS::Filespec::vmspath($to) if ON_VMS;
+
+        ### create the path if it doesn't exist yet ###
+        unless( -d $to ) {
+            eval { mkpath( $to ) };
+    
+            return $self->_error(loc("Could not create path '%1'",$to)) if $@;
+        }
     }
 
     ### set passive ftp if required ###
@@ -474,8 +494,24 @@ sub fetch {
 
             } else {
 
+                ### slurp mode?
+                if( ref $target and UNIVERSAL::isa( $target, 'SCALAR' ) ) {
+                    
+                    ### open the file
+                    open my $fh, "<$file" or do {
+                        $self->_error(
+                            loc("Could not open '%1': %2", $file, $!));
+                        return;                            
+                    };
+                    
+                    ### slurp
+                    $$target = do { local $/; <$fh> };
+                
+                } 
+
                 my $abs = File::Spec->rel2abs( $file );
                 return $abs;
+
             }
         }
     }
@@ -548,6 +584,171 @@ sub _lwp_fetch {
     }
 }
 
+### HTTP::Lite fetching ###
+sub _httplite_fetch {
+    my $self = shift;
+    my %hash = @_;
+
+    my ($to);
+    my $tmpl = {
+        to  => { required => 1, store => \$to }
+    };
+    check( $tmpl, \%hash ) or return;
+
+    ### modules required to download with lwp ###
+    my $use_list = {
+        'HTTP::Lite'    => '2.2',
+
+    };
+
+    # https://github.com/dagolden/cpanpm/compare/master...private%2Fuse-http-lite
+
+    if( can_load(modules => $use_list) ) {
+
+        my $uri = $self->uri;
+        my $retries = 0;
+
+        RETRIES: while ( $retries++ < 5 ) {
+
+          my $http = HTTP::Lite->new();
+          # Naughty naughty but there isn't any accessor/setter
+          $http->{timeout} = $TIMEOUT if $TIMEOUT;
+          $http->http11_mode(1);
+
+          my $fh = FileHandle->new;
+
+          unless ( $fh->open($to,'>') ) {
+            return $self->_error(loc(
+                 "Could not open '%1' for writing: %2",$to,$!));
+          }
+
+          $fh->autoflush(1);
+
+          binmode $fh;
+
+          my $rc = $http->request( $uri, sub { my ($self,$dref,$cbargs) = @_; local $\; print {$cbargs} $$dref }, $fh );
+
+          close $fh;
+
+          if ( $rc == 301 || $rc == 302 ) {
+              my $loc;
+              HEADERS: for ($http->headers_array) {
+                /Location: (\S+)/ and $loc = $1, last HEADERS;
+              }
+              #$loc or last; # Think we should squeal here.
+              if ($loc =~ m!^/!) {
+                $uri =~ s{^(\w+?://[^/]+)/.*$}{$1};
+                $uri .= $loc;
+              } 
+              else {
+                $uri = $loc;
+              }
+              next RETRIES;
+          }
+          elsif ( $rc == 200 ) {
+              return $to;
+          }
+          else {
+            return $self->_error(loc("Fetch failed! HTTP response: %1 [%2]",
+                        $rc, $http->status_message));
+          }
+
+        } # Loop for 5 retries.
+
+        return $self->_error("Fetch failed! Gave up after 5 tries");
+
+    } else {
+        $METHOD_FAIL->{'httplite'} = 1;
+        return;
+    }
+}
+
+### Simple IO::Socket::INET fetching ###
+sub _iosock_fetch {
+    my $self = shift;
+    my %hash = @_;
+
+    my ($to);
+    my $tmpl = {
+        to  => { required => 1, store => \$to }
+    };
+    check( $tmpl, \%hash ) or return;
+
+    my $use_list = {
+        'IO::Socket::INET' => '0.0',
+        'IO::Select'       => '0.0',
+    };
+
+    if( can_load(modules => $use_list) ) {
+        my $sock = IO::Socket::INET->new( 
+            PeerHost => $self->host,
+            ( $self->host =~ /:/ ? () : ( PeerPort => 80 ) ),
+        );
+
+        unless ( $sock ) {
+            return $self->_error(loc("Could not open socket to '%1', '%2'",$self->host,$!));
+        }
+
+        my $fh = FileHandle->new;
+
+        # Check open()
+
+        unless ( $fh->open($to,'>') ) {
+            return $self->_error(loc(
+                 "Could not open '%1' for writing: %2",$to,$!));
+        }
+
+        $fh->autoflush(1);
+        binmode $fh;
+
+        my $path = File::Spec::Unix->catfile( $self->path, $self->file );
+        my $req = "GET $path HTTP/1.0\x0d\x0aHost: " . $self->host . "\x0d\x0a\x0d\x0a";
+        $sock->send( $req );
+
+        my $select = IO::Select->new( $sock );
+
+        my $resp = '';
+        my $normal = 0;
+        while ( $select->can_read( $TIMEOUT || 60 ) ) {
+          my $ret = $sock->sysread( $resp, 4096, length($resp) );
+          if ( !defined $ret or $ret == 0 ) {
+            $select->remove( $sock );
+            $normal++;
+          }
+        }
+        close $sock;
+
+        unless ( $normal ) {
+            return $self->_error(loc("Socket timed out after '%1' seconds", ( $TIMEOUT || 60 )));
+        }
+
+        # Check the "response"
+        # Strip preceeding blank lines apparently they are allowed (RFC 2616 4.1)
+        $resp =~ s/^(\x0d?\x0a)+//;
+        # Check it is an HTTP response
+        unless ( $resp =~ m!^HTTP/(\d+)\.(\d+)!i ) {
+            return $self->_error(loc("Did not get a HTTP response from '%1'",$self->host));
+        }
+
+        # Check for OK
+        my ($code) = $resp =~ m!^HTTP/\d+\.\d+\s+(\d+)!i;
+        unless ( $code eq '200' ) {
+            return $self->_error(loc("Got a '%1' from '%2' expected '200'",$code,$self->host));
+        }
+
+        {
+          local $\;
+          print $fh +($resp =~ m/\x0d\x0a\x0d\x0a(.*)$/s )[0];
+        }
+        close $fh;
+        return $to;
+
+    } else {
+        $METHOD_FAIL->{'iosock'} = 1;
+        return;
+    }
+}
+
 ### Net::FTP fetching
 sub _netftp_fetch {
     my $self = shift;
@@ -1150,7 +1351,7 @@ Below is a mapping of what utilities will be used in what order
 for what schemes, if available:
 
     file    => LWP, lftp, file
-    http    => LWP, wget, curl, lftp, lynx
+    http    => LWP, HTTP::Lite, wget, curl, lftp, lynx, iosock
     ftp     => LWP, Net::FTP, wget, curl, lftp, ncftp, ftp
     rsync   => rsync
 
@@ -1162,6 +1363,9 @@ If a utility or module isn't available, it will be marked in a cache
 tried again. The C<fetch> method will only fail when all options are
 exhausted, and it was not able to retrieve the file.
 
+C<iosock> is a very limited L<IO::Socket::INET> based mechanism for
+retrieving C<http> schemed urls. It doesn't follow redirects for instance.
+
 A special note about fetching files from an ftp uri:
 
 By default, all ftp connections are done in passive mode. To change
@@ -1260,6 +1464,7 @@ Here's a quick mapping for the utilities/modules, and their names for
 the $BLACKLIST, $METHOD_FAIL and other internal functions.
 
     LWP         => lwp
+    HTTP::Lite  => httplite
     Net::FTP    => netftp
     wget        => wget
     lynx        => lynx
@@ -1268,6 +1473,7 @@ the $BLACKLIST, $METHOD_FAIL and other internal functions.
     curl        => curl
     rsync       => rsync
     lftp        => lftp
+    IO::Socket  => iosock
 
 =head1 FREQUENTLY ASKED QUESTIONS
 
